﻿Imports System.Runtime.InteropServices

#Region "ScreenSaver Class"

Public Module ScreenSaver
	Sub New()
	End Sub

	' Signatures for unmanaged calls
	<DllImport("user32.dll", CharSet:=CharSet.Auto)>
	Private Function SystemParametersInfo(ByVal uAction As Integer,
				ByVal uParam As Integer, ByRef lpvParam As Integer,
				ByVal flags As Integer) As Boolean
	End Function

	<DllImport("user32.dll", CharSet:=CharSet.Auto)>
	Private Function PostMessage(ByVal hWnd As IntPtr, ByVal wMsg As Integer,
				ByVal wParam As Integer, ByVal lParam As Integer) As Integer
	End Function

	<DllImport("user32.dll", CharSet:=CharSet.Auto)>
	Private Function OpenDesktop(ByVal hDesktop As String, ByVal Flags As Integer,
				ByVal Inherit As Boolean, ByVal DesiredAccess As UInteger) As IntPtr
	End Function

	<DllImport("user32.dll", CharSet:=CharSet.Auto)>
	Private Function CloseDesktop(ByVal hDesktop As IntPtr) As Boolean
	End Function

	<DllImport("user32.dll", CharSet:=CharSet.Auto)>
	Private Function EnumDesktopWindows(ByVal hDesktop As IntPtr,
				ByVal callback As EnumDesktopWindowsProc,
				ByVal lParam As IntPtr) As Boolean
	End Function

	<DllImport("user32.dll", CharSet:=CharSet.Auto)>
	Private Function IsWindowVisible(ByVal hWnd As IntPtr) As Boolean
	End Function

	<DllImport("user32.dll", CharSet:=CharSet.Auto)>
	Public Function GetForegroundWindow() As IntPtr
	End Function

	Private Declare Function SystemParametersInfoPointer _
		Lib "user32" Alias "SystemParametersInfoA" (ByVal uiAction As Int32,
																	ByVal uiParam As Int32,
																	ByRef pvParam As Int32,
																	ByVal fWinIni As Int32) As Int32

	' Callbacks
	Private Delegate Function EnumDesktopWindowsProc(ByVal hDesktop As IntPtr,
																ByVal lParam As IntPtr) As Boolean
	' Constants
	Public Const SPI_GETSCREENSAVERACTIVE As Integer = 16
	Public Const SPI_SETSCREENSAVERACTIVE As Integer = 17
	Public Const SPI_GETSCREENSAVERTIMEOUT As Integer = 14
	Public Const SPI_SETSCREENSAVERTIMEOUT As Integer = 15
	Public Const SPI_GETSCREENSAVERRUNNING As Integer = 114
	Public Const SPIF_SENDWININICHANGE As Integer = 2
	Public Const SC_SCREENSAVE As UInteger = &HF140

	Private Enum SpecialHandles
		HWND_DESKTOP = &H0
		HWND_BROADCAST = &HFFFF
	End Enum

	Private Const DESKTOP_WRITEOBJECTS As UInteger = &H80
	Private Const DESKTOP_READOBJECTS As UInteger = &H1
	Private Const WM_CLOSE As Integer = 16

	Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" _
		(ByVal hwnd As IntPtr,
		ByVal wMsg As Integer,
		ByVal wParam As Integer,
		ByVal lParam As IntPtr) As Integer

	Private Const WM_SYSCOMMAND = &H112


	''' <summary>
	'''starten des Screensavers
	''' </summary> 
	Public Sub TurnOnScreenSaver()
		Dim screenSaverRunning As Integer = -1
		Dim ok As Integer =
			SystemParametersInfo(SPI_GETSCREENSAVERRUNNING, 0, screenSaverRunning, 0)
		'wenn ok = 0, dann läuft SCR
		SendMessage(New IntPtr(CInt(SpecialHandles.HWND_BROADCAST)),
						WM_SYSCOMMAND, SC_SCREENSAVE, 0)
	End Sub

	''' <summary>
	''' um screensaver zu aktivieren: active=true, sonst active=false
	''' </summary> 
	Public Function ToggleScreenSaverActive(ByVal Active As Boolean) As Boolean
		Dim lActiveFlag, retval As Long
		lActiveFlag = If(Active, 1, 0)
		retval = SystemParametersInfo(SPI_SETSCREENSAVERACTIVE, lActiveFlag, 0, 0)
		Return (retval > 0)
	End Function

	'ermitteln der Screensaver Startzeit
	Public Function GetScreenSaverActivationTime() As Integer
		Dim ptSaverTimeOut As Integer
		Dim Result As Integer
		Result = SystemParametersInfoPointer(SPI_GETSCREENSAVERTIMEOUT, 0, ptSaverTimeOut, 0)
		If Result <> 0 Then
			Return ptSaverTimeOut
		Else
			Throw New Exception("Error...")
		End If
	End Function

	' Returns TRUE if the screen saver is active (enabled, but not necessarily running).
	Public Function GetScreenSaverActive() As Boolean
		Dim isActive As Boolean = False
		SystemParametersInfo(SPI_GETSCREENSAVERACTIVE, 0, isActive, 0)
		Return isActive
	End Function

	' Pass in TRUE(1) to activate or FALSE(0) to deactivate the screen saver.
	Public Sub SetScreenSaverActive(ByVal Active As Integer)
		Dim nullVar As Integer = 0
		SystemParametersInfo(SPI_SETSCREENSAVERACTIVE, Active, nullVar,
									SPIF_SENDWININICHANGE)
	End Sub

	' Returns the screen saver timeout setting, in seconds
	Public Function GetScreenSaverTimeout() As Int32
		Dim value As Int32 = 0
		SystemParametersInfo(SPI_GETSCREENSAVERTIMEOUT, 0, value, 0)
		Return value
	End Function

	' Pass in the number of seconds to set the screen saver timeout value.
	Public Sub SetScreenSaverTimeout(ByVal Value As Int32)
		Dim nullVar As Integer = 0
		SystemParametersInfo(SPI_SETSCREENSAVERTIMEOUT, Value, nullVar, SPIF_SENDWININICHANGE)
	End Sub

	' Returns TRUE if the screen saver is actually running
	Public Function GetScreenSaverRunning() As Boolean
		Dim isRunning As Boolean = False
		SystemParametersInfo(SPI_GETSCREENSAVERRUNNING, 0, isRunning, 0)
		Return isRunning
	End Function

	Private Function KillScreenSaverFunc(ByVal hWnd As IntPtr,
													 ByVal lParam As IntPtr) As Boolean
		If IsWindowVisible(hWnd) Then PostMessage(hWnd, WM_CLOSE, 0, 0)
		Return True
	End Function

End Module

#End Region
